import os
from typing import List
import glob
from tqdm import tqdm

from blueness import module
from bluer_objects import objects, file
from bluer_objects.env import abcli_path_git

from bluer_sandbox import NAME
from bluer_sandbox.logger import logger


NAME = module.name(__file__, NAME)


def publish(
    object_name: str,
    list_of_extensions: List[str],
    prefix: str = "",
    log: bool = True,
) -> bool:
    logger.info(
        "{}.publish: {}/{}.* for {}".format(
            NAME,
            object_name,
            prefix,
            ", ".join(list_of_extensions),
        )
    )

    for extension in tqdm(list_of_extensions):
        for filename in glob.glob(
            objects.path_of(
                filename=f"{prefix}*.{extension}",
                object_name=object_name,
            )
        ):
            published_filename = os.path.join(
                abcli_path_git,
                "assets",
                object_name,
                file.name_and_extension(filename),
            )

            if extension in ["png", "jpg", "jpeg", "gif", "txt"]:
                if not file.copy(
                    filename,
                    published_filename,
                    log=log,
                ):
                    return False

    logger.info(f"🔗  https://github.com/kamangir/assets/tree/main/{object_name}")

    return True



import os

from bluer_options.help.functions import get_help
from bluer_objects import file, README

from bluer_sandbox import NAME, VERSION, ICON, REPO_NAME
from bluer_sandbox.help.functions import help_functions


items = README.Items(
    [
        {
            "name": "offline LLM",
            "description": "using [llama.cpp](https://github.com/ggerganov/llama.cpp).",
            "marquee": "https://user-images.githubusercontent.com/1991296/230134379-7181e485-c521-4d23-a0d6-f7b3b61ba524.png",
            "url": "./bluer_sandbox/docs/offline_llm.md",
        },
    ]
)


def build():
    return all(
        README.build(
            items=readme.get("items", []),
            cols=readme.get("cols", 3),
            path=os.path.join(file.path(__file__), readme["path"]),
            ICON=ICON,
            NAME=NAME,
            VERSION=VERSION,
            REPO_NAME=REPO_NAME,
            help_function=lambda tokens: get_help(
                tokens,
                help_functions,
                mono=True,
            ),
        )
        for readme in [
            {
                "path": "..",
                "items": items,
                "cols": 1,
            },
            {
                "path": "docs/offline_llm.md",
            },
            # aliases
            {
                "path": "docs/aliases/assets.md",
            },
            {
                "path": "docs/aliases/docker.md",
            },
            {
                "path": "docs/aliases/notebooks.md",
            },
            {
                "path": "docs/aliases/offline_llm.md",
            },
        ]
    )



from typing import List

from bluer_options.terminal import show_usage, xtra

from bluer_sandbox.help.offline_llm.model import help_functions as help_model


def help_build(
    tokens: List[str],
    mono: bool,
) -> str:
    options = xtra("dryrun", mono=mono)

    return show_usage(
        [
            "@offline_llm",
            "build",
            f"[{options}]",
        ],
        "build offline_llm.",
        mono=mono,
    )


def help_chat(
    tokens: List[str],
    mono: bool,
) -> str:
    options = xtra("download_model,tiny,~upload", mono=mono)

    return show_usage(
        [
            "@offline_llm",
            "chat",
            f"[{options}]",
            "[-|<object-name>]",
        ],
        "chat with offline_llm.",
        mono=mono,
    )


def help_create_env(
    tokens: List[str],
    mono: bool,
) -> str:
    return show_usage(
        [
            "@plugin",
            "create_env",
        ],
        "create env.",
        mono=mono,
    )


def help_prompt(
    tokens: List[str],
    mono: bool,
) -> str:
    options = xtra("download_model,tiny,~upload", mono=mono)

    return show_usage(
        [
            "@offline_llm",
            "prompt",
            f"[{options}]",
            '"<prompt>"',
            "[-|<object-name>]",
        ],
        '"<prompt>" -> offline_llm.',
        mono=mono,
    )


help_functions = {
    "build": help_build,
    "chat": help_chat,
    "create_env": help_create_env,
    "model": help_model,
    "prompt": help_prompt,
}



from typing import List

from bluer_options.terminal import show_usage, xtra


def help_browse(
    tokens: List[str],
    mono: bool,
) -> str:
    options = xtra("~public", mono=mono)

    return show_usage(
        [
            "@docker",
            "browse",
            f"[{options}]",
        ],
        "browse docker-hub.",
        mono=mono,
    )


def help_build(
    tokens: List[str],
    mono: bool,
) -> str:
    options = xtra("dryrun,no_cache,~push,run,verbose", mono=mono)

    return show_usage(
        [
            "@docker",
            "build",
            f"[{options}]",
        ],
        "build the docker image.",
        mono=mono,
    )


def help_clear(
    tokens: List[str],
    mono: bool,
) -> str:
    return show_usage(
        [
            "@docker",
            "clear",
        ],
        "clear docker.",
        mono=mono,
    )


def help_eval(
    tokens: List[str],
    mono: bool,
) -> str:
    options = xtra("cat,dryrun,verbose", mono=mono)

    return show_usage(
        [
            "@docker",
            "eval",
            f"[{options}]",
            "<command-line>",
        ],
        "run <command-line> through the docker image.",
        mono=mono,
    )


def help_push(
    tokens: List[str],
    mono: bool,
) -> str:
    return show_usage(
        [
            "@docker",
            "push",
        ],
        "push the docker image.",
        mono=mono,
    )


def help_run(
    tokens: List[str],
    mono: bool,
) -> str:
    options = xtra("dryrun", mono=mono)

    return show_usage(
        [
            "@docker",
            "run",
            f"[{options}]",
        ],
        "run the docker image.",
        mono=mono,
    )


def help_seed(
    tokens: List[str],
    mono: bool,
) -> str:
    return show_usage(
        [
            "@docker",
            "seed",
        ],
        "seed docker 🌱 .",
        mono=mono,
    )


help_functions = {
    "browse": help_browse,
    "build": help_build,
    "clear": help_clear,
    "eval": help_eval,
    "push": help_push,
    "run": help_run,
    "seed": help_seed,
}



from bluer_ai.help.generic import help_functions as generic_help_functions
from bluer_sandbox.help.assets import help_functions as help_assets
from bluer_sandbox.help.docker import help_functions as help_docker
from bluer_sandbox.help.notebooks import help_functions as help_notebooks
from bluer_sandbox.help.offline_llm import help_functions as help_offline_llm

from bluer_sandbox import ALIAS


help_functions = generic_help_functions(plugin_name=ALIAS)

help_functions.update(
    {
        "assets": help_assets,
        "docker": help_docker,
        "notebooks": help_notebooks,
        "offline_llm": help_offline_llm,
    }
)
